@@ -34,8 +34,7 @@ A Sphinx directive to specify that a module has extra requirements, and show how
 import inspect
 import re
 import string
-import typing
-from typing import Any, Dict, Union
+from typing import Any, Callable, Dict, Iterator, List, Mapping, Pattern, Tuple, Type, Union
 
 # 3rd party
 from docutils.nodes import document
@@ -44,59 +43,71 @@ from sphinx.application import Sphinx
 from sphinx.parsers import RSTParser
 from sphinx.util.inspect import signature as Signature
 
+try:
+	# 3rd party
+	import attr
+except ImportError:  # pragma: no cover
+	# attrs is used in a way that it is only required in situations
+	# where it is available to import, so its fine to do this.
+	pass
+
 __author__: str = "Dominic Davis-Foster"
 __copyright__: str = "2020 Dominic Davis-Foster"
 
 __license__: str = "MIT"
-__version__: str = "0.0.11"
+__version__: str = "0.1.0"
 __email__: str = "dominic@davis-foster.co.uk"
 
-__all__ = ["process_docstring", "process_default_format", "setup"]
+__all__ = [
+		"process_docstring",
+		"process_default_format",
+		"setup",
+		"get_class_defaults",
+		"get_function_defaults",
+		"default_regex",
+		"no_default_regex",
+		"get_arguments",
+		]
 
-default_regex: typing.Pattern = re.compile(r"^:(default|Default) ")
-no_default_regex: typing.Pattern = re.compile(r"^:(No|no)[-_](default|Default) ")
+#: Regular expression to match default values declared in docstrings.
+default_regex: Pattern = re.compile(r"^:(default|Default) ")
+
+#: Regular expression to match flags in docstrings to suppress default values.
+no_default_regex: Pattern = re.compile(r"^:(No|no)[-_](default|Default) ")
 
 
-def process_docstring(app: Sphinx, what, name, obj, options, lines: typing.List[str]) -> None:
+def process_docstring(
+		app: Sphinx, what: str, name: str, obj: Any, options: Dict[str, Any], lines: List[str]
+		) -> None:
 	"""
 	Add default values to the docstring.
 
-	:param app:
+	:param app: The Sphinx app.
 	:param what:
-	:type what:
-	:param name:
-	:type name:
-	:param obj:
-	:type obj:
-	:param options:
-	:type options:
+	:param name: The name of the object being documented.
+	:param obj: The object being documented.
+	:param options: Mapping of autodoc options to values.
 	:param lines: List of strings representing the current contents of the docstring.
 	"""
 
-	# Size varies depending on docutils config
-	a_tab = " " * app.config.docutils_tab_width  # type: ignore
-
 	if isinstance(obj, property):
 		return None
 
+	# Size varies depending on docutils config
+	a_tab = " " * app.config.docutils_tab_width  # type: ignore
+
 	if callable(obj):
+
+		default_getter: Union[Callable[[Type], _defaults], Callable[[Callable], _defaults]]
+
 		if inspect.isclass(obj):
-			obj = getattr(obj, "__init__")
-
-		obj = inspect.unwrap(obj)
-
-		try:
-			signature = Signature(obj)
-		except ValueError:
-			return None
+			default_getter = get_class_defaults
+		else:
+			default_getter = get_function_defaults
 
 		default_description_format: str = app.config.default_description_format  # type: ignore
 
-		for argname, param in signature.parameters.items():
-			if argname.endswith('_'):
-				argname = f"{argname[:-1]}\\_"
-
-			default_value = param.default
+		for argname, default_value in default_getter(obj):
 			formatted_annotation = None
 
 			# Get the default value from the signature
@@ -168,6 +179,69 @@ def process_docstring(app: Sphinx, what, name, obj, options, lines: typing.List[
 	return None
 
 
+_defaults = Iterator[Tuple[str, Any]]
+
+
+def get_class_defaults(obj: Type) -> _defaults:
+	"""
+	Obtains the default values for the arguments of a class.
+
+	:param obj: The class.
+
+	:return: An iterator of 2-element tuples comprising the argument name and its default value.
+	"""
+
+	for argname, param in get_arguments(getattr(obj, "__init__")).items():
+		if argname.endswith('_'):
+			argname = f"{argname[:-1]}\\_"
+
+		default_value = param.default
+
+		if hasattr(obj, "__attrs_attrs__"):
+			# Special casing for attrs classes
+			if default_value is attr.NOTHING:
+				for value in obj.__attrs_attrs__:
+					if value.name == argname and isinstance(value.default, attr.Factory):  # type: ignore
+						default_value = value.default.factory()
+
+		yield argname, default_value
+
+	return None
+
+
+def get_function_defaults(obj: Callable) -> _defaults:
+	"""
+	Obtains the default values for the arguments of a function.
+
+	:param obj: The function.
+
+	:return: An iterator of 2-element tuples comprising the argument name and its default value.
+	"""
+
+	for argname, param in get_arguments(obj).items():
+		if argname.endswith('_'):
+			argname = f"{argname[:-1]}\\_"
+
+		yield argname, param.default
+
+	return None
+
+
+def get_arguments(obj: Callable) -> Mapping[str, inspect.Parameter]:
+	"""
+	Returns a dictionary mapping argument names to parameters/arguments for a function.
+
+	:param obj: A function (can be the ``__init__`` method of a class).
+	"""
+
+	try:
+		signature = Signature(inspect.unwrap(obj))
+	except ValueError:  # pragma: no cover
+		return {}
+
+	return signature.parameters
+
+
 def process_default_format(app: Sphinx) -> None:
 	"""
 	Prepare the formatting of the default value.
@@ -209,7 +283,7 @@ def setup(app: Sphinx) -> Dict[str, Any]:
 	# Hack to get the docutils tab size, as there doesn't appear to be any other way
 	class CustomRSTParser(RSTParser):
 
-		def parse(self, inputstring: Union[str, StringList], document: document) -> None:
+		def parse(self, inputstring: Union[str, StringList], document: document) -> None:  # pragma: no cover
 			app.config.docutils_tab_width = document.settings.tab_width  # type: ignore
 			super().parse(inputstring, document)
 
